from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, TYPE_CHECKING

from attrs import define as _attrs_define
from attrs import field as _attrs_field


if TYPE_CHECKING:
    from ..models.voice_info import VoiceInfo


T = TypeVar("T", bound="VoicesResponse")


@_attrs_define
class VoicesResponse:
    """Response containing list of voices

    Attributes:
        voices (list[VoiceInfo]):
        total (int):
    """

    voices: list[VoiceInfo]
    total: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        voices = []
        for voices_item_data in self.voices:
            voices_item = voices_item_data.to_dict()
            voices.append(voices_item)

        total = self.total

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "voices": voices,
                "total": total,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.voice_info import VoiceInfo

        d = dict(src_dict)
        voices = []
        _voices = d.pop("voices")
        for voices_item_data in _voices:
            voices_item = VoiceInfo.from_dict(voices_item_data)

            voices.append(voices_item)

        total = d.pop("total")

        voices_response = cls(
            voices=voices,
            total=total,
        )

        voices_response.additional_properties = d
        return voices_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
